package nl.kallestruik.gitlabrepostats.objects

import kotlinx.serialization.KSerializer
import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable
import kotlinx.serialization.descriptors.PrimitiveKind
import kotlinx.serialization.descriptors.PrimitiveSerialDescriptor
import kotlinx.serialization.descriptors.SerialDescriptor
import kotlinx.serialization.encoding.Decoder
import kotlinx.serialization.encoding.Encoder
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter

@Serializable
data class GitlabIssue(
    val iid: Long,
    val state: String,
    val assignees: List<GitlabUser>,
    @SerialName("closed_at")
    @Serializable(with = LocalDateTimeAsISOStringSerializer::class)
    val closedAt: LocalDateTime?,
    @SerialName("time_stats")
    val timeStats: GitlabTimeStats,
)

object LocalDateTimeAsISOStringSerializer : KSerializer<LocalDateTime> {
    override val descriptor: SerialDescriptor = PrimitiveSerialDescriptor("Date", PrimitiveKind.STRING)
    override fun serialize(encoder: Encoder, value: LocalDateTime) = encoder.encodeString(value.toString())
    override fun deserialize(decoder: Decoder): LocalDateTime = LocalDateTime.parse(decoder.decodeString(), DateTimeFormatter.ISO_OFFSET_DATE_TIME)
}

@Serializable
data class GitlabTimeStats(
    @SerialName("time_estimate")
    val timeEstimate: Int?,
    @SerialName("total_time_spent")
    val totalTimeSpent: Int?,
    @SerialName("human_time_estimate")
    val humanTimeEstimate: String?,
    @SerialName("human_total_time_spent")
    val humanTotalTimeSpent: String?,
)
